# -*- coding: utf-8 -*-
from design.plone.policy.utils import folderSubstructureGenerator
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return ["design.plone.policy:uninstall"]


def post_install(context):
    """Post install script"""
    folderSubstructureGenerator(
        title="Amministrazione", types=("Document", "Image", "File", "Link")
    )
    folderSubstructureGenerator(
        title="Servizi", types=("Document", "Image", "File", "Link")
    )
    folderSubstructureGenerator(
        title="Novità", types=("Document", "Image", "File", "Link")
    )
    folderSubstructureGenerator(
        title="Documenti e dati",
        types=(
            "Document",
            "Image",
            "File",
            "Link",
            "Documento",
            "CartellaModulistica",
        ),
    )
    folderSubstructureGenerator(
        title="Argomenti", types=("Document", "Image", "File", "Link")
    )


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-
from AccessControl.unauthorized import Unauthorized
from plone.restapi.services import Service
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse
from plone import api

SECTION_IDS = ["amministrazione", "servizi", "novita", "documenti-e-dati"]


@implementer(IPublishTraverse)
class SearchFiltersGet(Service):
    def __init__(self, context, request):
        super(SearchFiltersGet, self).__init__(context, request)

    def get_basic_data(self, item):
        return {
            "@id": item.absolute_url(),
            "title": item.Title(),
            "description": item.description,
            "@type": item.portal_type,
            "path": item.virtual_url_path(),
            "UID": item.UID(),
        }

    def reply(self):
        portal_path = "/".join(api.portal.get().getPhysicalPath())
        sections = {}
        topics = []

        for section_id in SECTION_IDS:
            section_path = "{portal}/{id}".format(
                portal=portal_path, id=section_id
            )
            try:
                section = api.content.get(section_path)
            except Unauthorized:
                # private folder
                continue
            if not section:
                continue
            sections[section_id] = self.get_basic_data(item=section)
            sections[section_id]["items"] = []
            for children in section.listFolderContents():
                sections[section_id]["items"].append(
                    self.get_basic_data(item=children)
                )
        for argument in api.content.find(portal_type="Pagina Argomento"):
            topics.append(self.get_basic_data(argument.getObject()))
        res = {"sections": sections, "topics": topics}
        return res



# -*- coding: utf-8 -*-
from design.plone.policy.testing import DESIGN_PLONE_POLICY_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class TestInitialStructureCreation(unittest.TestCase):

    layer = DESIGN_PLONE_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_first_level_created(self):
        self.assertEqual(
            [x.id for x in self.portal.getFolderContents()],
            [
                "amministrazione",
                "servizi",
                "novita",
                "documenti-e-dati",
                "argomenti",
            ],
        )

    def test_amministrazione_section(self):

        amministrazione = self.portal["amministrazione"]
        self.assertEqual(amministrazione.constrain_types_mode, 1)
        self.assertEqual(
            amministrazione.locally_allowed_types,
            ("Document", "Image", "File", "Link"),
        )

        self.assertEqual(
            amministrazione.keys(),
            [
                "politici",
                "personale-amministrativo",
                "organi-di-governo",
                "aree-amministrative",
                "uffici",
                "enti-e-fondazioni",
                "luoghi",
            ],
        )

        self.assertEqual(amministrazione["politici"].portal_type, "Document")
        self.assertEqual(amministrazione["politici"].constrain_types_mode, 1)
        self.assertEqual(
            amministrazione["politici"].locally_allowed_types,
            ("Document", "Image", "File", "Link"),
        )

        self.assertEqual(
            amministrazione["personale-amministrativo"].portal_type, "Document"
        )
        self.assertEqual(
            amministrazione["personale-amministrativo"].constrain_types_mode, 1
        )
        self.assertEqual(
            amministrazione["personale-amministrativo"].locally_allowed_types,
            ("Persona",),
        )

        self.assertEqual(
            amministrazione["organi-di-governo"].portal_type, "Document"
        )
        self.assertEqual(
            amministrazione["organi-di-governo"].constrain_types_mode, 1
        )
        self.assertEqual(
            amministrazione["organi-di-governo"].locally_allowed_types,
            ("UnitaOrganizzativa",),
        )

        self.assertEqual(
            amministrazione["aree-amministrative"].portal_type, "Document"
        )
        self.assertEqual(
            amministrazione["aree-amministrative"].constrain_types_mode, 1
        )
        self.assertEqual(
            amministrazione["aree-amministrative"].locally_allowed_types,
            ("UnitaOrganizzativa",),
        )

        self.assertEqual(amministrazione["uffici"].portal_type, "Document")
        self.assertEqual(amministrazione["uffici"].constrain_types_mode, 1)
        self.assertEqual(
            amministrazione["uffici"].locally_allowed_types,
            ("UnitaOrganizzativa",),
        )

        self.assertEqual(
            amministrazione["enti-e-fondazioni"].portal_type, "Document"
        )
        self.assertEqual(
            amministrazione["enti-e-fondazioni"].constrain_types_mode, 1
        )
        self.assertEqual(
            amministrazione["enti-e-fondazioni"].locally_allowed_types,
            ("UnitaOrganizzativa",),
        )

        self.assertEqual(amministrazione["luoghi"].portal_type, "Document")
        self.assertEqual(amministrazione["luoghi"].constrain_types_mode, 1)
        self.assertEqual(
            amministrazione["luoghi"].locally_allowed_types, ("Venue",)
        )

    def test_servizi_section(self):

        servizi = self.portal["servizi"]
        self.assertEqual(servizi.constrain_types_mode, 1)
        self.assertEqual(
            servizi.locally_allowed_types,
            ("Document", "Image", "File", "Link"),
        )
        self.assertEqual(
            servizi.keys(),
            [
                "anagrafe-e-stato-civile",
                "cultura-e-tempo-libero",
                "vita-lavorativa",
                "attivita-produttive-e-commercio",
                "appalti-pubblici",
                "catasto-e-urbanistica",
                "turismo",
                "mobilita-e-trasporti",
                "educazione-e-formazione",
                "giustizia-e-sicurezza-pubblica",
                "tributi-e-finanze",
                "ambiente",
                "salute-benessere-e-assistenza",
                "autorizzazioni",
                "agricoltura",
            ],
        )

        for child in servizi.listFolderContents():
            self.assertEqual(child.portal_type, "Document")
            self.assertEqual(child.constrain_types_mode, 1)
            self.assertEqual(
                child.locally_allowed_types,
                ("Document", "Image", "File", "Link"),
            )

    def test_novita_section(self):

        folder = self.portal["novita"]
        self.assertEqual(folder.constrain_types_mode, 1)
        self.assertEqual(
            folder.locally_allowed_types, ("Document", "Image", "File", "Link")
        )
        self.assertEqual(folder.keys(), ["notizie", "comunicati", "eventi"])

        for child in folder.listFolderContents():
            self.assertEqual(child.portal_type, "Document")
            self.assertEqual(child.constrain_types_mode, 1)
            if child.getId() == "eventi":
                self.assertEqual(
                    child.locally_allowed_types,
                    ("Document", "Image", "File", "Link"),
                )
            else:
                self.assertEqual(child.locally_allowed_types, ("News Item",))

    def test_documenti_e_dati_section(self):

        folder = self.portal["documenti-e-dati"]
        self.assertEqual(folder.constrain_types_mode, 1)
        self.assertEqual(
            folder.locally_allowed_types,
            (
                "Document",
                "Image",
                "File",
                "Link",
                "Documento",
                "CartellaModulistica",
            ),
        )
        self.assertEqual(
            folder.keys(),
            [
                "documenti-albo-pretorio",
                "modulistica",
                "documenti-funzionamento-interno",
                "atti-normativi",
                "accordi-tra-enti",
                "documenti-attivita-politica",
                "documenti-tecnici-di-supporto",
                "istanze",
                "dataset",
            ],
        )

        for child in folder.listFolderContents():
            self.assertEqual(child.portal_type, "Document")
            self.assertEqual(child.constrain_types_mode, 1)
            self.assertEqual(
                child.locally_allowed_types,
                (
                    "Document",
                    "Image",
                    "File",
                    "Link",
                    "Documento",
                    "CartellaModulistica",
                ),
            )

    def test_argomenti_section(self):
        folder = self.portal["argomenti"]
        self.assertEqual(folder.portal_type, "Document")
        self.assertEqual(folder.constrain_types_mode, 1)
        self.assertEqual(folder.locally_allowed_types, ("Pagina Argomento",))



# -*- coding: utf-8 -*-
from email.message import EmailMessage
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from plone.registry.interfaces import IRegistry
from plone.restapi.deserializer import json_body
from plone.restapi.services import Service
from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from smtplib import SMTPException
from zExceptions import BadRequest
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides

import codecs
import six


class SendActionFormPost(Service):
    def reply(self):
        data = json_body(self.request)

        block_id = data.get("block_id", "")
        if not block_id:
            raise BadRequest("block_id mancante")

        block_data = self.get_block_data(block_id=block_id)
        if not block_data:
            raise BadRequest(
                'Blocco di tipo "form" con id "{}" non trovato'.format(  # noqa
                    block_id
                )
            )
        message = data.get("message", "")
        subject = data.get("subject", "") or block_data.get(
            "default_subject", ""
        )
        mfrom = data.get("from", "") or block_data.get("default_from", "")

        if not message or not subject or not mfrom:
            raise BadRequest(
                "Campo obbligatorio tra message, subject e from mancante".format(  # noqa
                    block_id
                ),
            )
        portal = api.portal.get()
        overview_controlpanel = getMultiAdapter(
            (portal, self.request), name="overview-controlpanel"
        )
        if overview_controlpanel.mailhost_warning():
            raise BadRequest("MailHost is not configured.")

        # Disable CSRF protection
        alsoProvides(self.request, IDisableCSRFProtection)

        registry = getUtility(IRegistry)
        mail_settings = registry.forInterface(IMailSchema, prefix="plone")
        mto = block_data.get("to", mail_settings.email_from_address)
        encoding = registry.get("plone.email_charset", "utf-8")
        host = api.portal.get_tool(name="MailHost")

        msg = EmailMessage()
        msg.set_content(message)
        msg["Subject"] = subject
        msg["From"] = mfrom
        msg["To"] = mto
        msg["Reply-To"] = mfrom

        self.manage_attachments(data=data, msg=msg)

        try:
            host.send(
                msg, charset=encoding,
            )

        except (SMTPException, RuntimeError):
            plone_utils = api.portal.get_tool(name="plone_utils")
            exception = plone_utils.exceptionString()
            message = "Unable to send mail: {}".format(exception)

            self.request.response.setStatus(500)
            return dict(
                error=dict(type="InternalServerError", message=message)
            )

        return self.reply_no_content()

    def get_block_data(self, block_id):
        blocks = getattr(self.context, "blocks", {})
        if not blocks:
            return {}
        for id, block in blocks.items():
            if id != block_id:
                continue
            block_type = block.get("@type", "")
            if block_type != "form":
                continue
            return block
        return {}

    def manage_attachments(self, data, msg):
        attachments = data.get("attachments", {})
        if not attachments:
            return []
        for key, value in attachments.items():
            content_type = "application/octet-stream"
            filename = None
            if isinstance(value, dict):
                file_data = value.get("data", "")
                if not file_data:
                    continue
                content_type = value.get("content-type", content_type)
                filename = value.get("filename", filename)
                if isinstance(file_data, six.text_type):
                    file_data = file_data.encode("utf-8")
                if "encoding" in value:
                    file_data = codecs.decode(file_data, value["encoding"])
                if isinstance(file_data, six.text_type):
                    file_data = file_data.encode("utf-8")
            else:

                file_data = value
            for attachment in attachments:
                msg.add_attachment(
                    file_data,
                    maintype=content_type,
                    subtype=content_type,
                    filename=filename,
                )



# -*- coding: utf-8 -*-
from plone import api
from plone.i18n.normalizer.interfaces import IIDNormalizer
from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from zope.component import getUtility


TASSONOMIA_SERVIZI = [
    "Anagrafe e stato civile",
    "Cultura e tempo libero",
    "Vita lavorativa",
    "Attività produttive e commercio",
    "Appalti pubblici",
    "Catasto e urbanistica",
    "Turismo",
    "Mobilità e trasporti",
    "Educazione e formazione",
    "Giustizia e sicurezza pubblica",
    "Tributi e finanze",
    "Ambiente",
    "Salute, benessere e assistenza",
    "Autorizzazioni",
    "Agricoltura",
]

TASSONOMIA_DOCUMENTI = [
    "Documenti albo pretorio",
    "Modulistica",
    "Documenti funzionamento interno",
    "Atti normativi",
    "Accordi tra enti",
    "Documenti attività politica",
    "Documenti (tecnici) di supporto",
    "Istanze",
    "Dataset",
]

TASSONOMIA_NEWS = ["Notizie", "Comunicati", "Eventi"]


def folderSubstructureGenerator(title, types=[]):
    container = api.portal.get()

    normalizer = getUtility(IIDNormalizer)
    safe_id = normalizer.normalize(title)

    root_path = '/'.join(container.getPhysicalPath())
    path = root_path + '/' + safe_id

    if api.content.get(path=path):
        return

    tree_root = api.content.create(container=container, type="Document", title=title)
    api.content.transition(obj=tree_root, transition="publish")
    if types:
        restrict_types(context=tree_root, types=types)

    if title == "Servizi":
        for ts in TASSONOMIA_SERVIZI:
            folder = api.content.create(container=tree_root, type="Document", title=ts)
            # temporary disabled
            # restrict_types(context=folder, types=("Servizio",))

    elif title == "Documenti e dati":
        for td in TASSONOMIA_DOCUMENTI:
            folder = api.content.create(container=tree_root, type="Document", title=td)

    elif title == "Novità":
        for tn in TASSONOMIA_NEWS:
            folder = api.content.create(container=tree_root, type="Document", title=tn)

            if tn == "Eventi":
                # temporary disabled
                # restrict_types(context=folder, types=("Event",))
                pass
            else:
                restrict_types(context=folder, types=("News Item",))

    elif title == "Amministrazione":
        api.content.create(type="Document", title="Politici", container=tree_root)
        # restrict_types(context=tree_root['politici'], types=("Persona",))

        api.content.create(
            type="Document",
            title="Personale Amministrativo",
            container=tree_root,
        )
        restrict_types(
            context=tree_root["personale-amministrativo"], types=("Persona",)
        )

        api.content.create(
            type="Document", title="Organi di governo", container=tree_root
        )
        restrict_types(
            context=tree_root["organi-di-governo"],
            types=("UnitaOrganizzativa",),
        )

        api.content.create(
            type="Document", title="Aree amministrative", container=tree_root
        )
        restrict_types(
            context=tree_root["aree-amministrative"],
            types=("UnitaOrganizzativa",),
        )

        api.content.create(type="Document", title="Uffici", container=tree_root)
        restrict_types(context=tree_root["uffici"], types=("UnitaOrganizzativa",))

        api.content.create(
            type="Document", title="Enti e fondazioni", container=tree_root
        )
        restrict_types(
            context=tree_root["enti-e-fondazioni"],
            types=("UnitaOrganizzativa",),
        )

        api.content.create(type="Document", title="Luoghi", container=tree_root)
        restrict_types(context=tree_root["luoghi"], types=("Venue",))

    elif title == "Argomenti":
        restrict_types(context=tree_root, types=("Pagina Argomento",))


def restrict_types(context, types):
    constraints = ISelectableConstrainTypes(context)
    constraints.setConstrainTypesMode(1)
    constraints.setLocallyAllowedTypes(types)



# -*- coding: utf-8 -*-
from design.plone.policy.testing import (
    DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession

import unittest


class SearchFiltersAPITest(unittest.TestCase):

    layer = DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def tearDown(self):
        self.api_session.close()

    def test_endpoint_exists(self):
        response = self.api_session.get("/@search-filters")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers.get("Content-Type"), "application/json"
        )

    def test_endpoint_return_list_of_sections(self):
        response = self.api_session.get("/@search-filters").json()

        self.assertIn('sections', response)
        sections = response['sections']
        self.assertIn('amministrazione', sections)
        self.assertIn('documenti-e-dati', sections)
        self.assertIn('novita', sections)
        self.assertIn('servizi', sections)

    def test_endpoint_return_list_of_topics_empty_if_no_topics(self):
        response = self.api_session.get("/@search-filters").json()

        self.assertIn('topics', response)
        self.assertEqual(response['topics'], [])

    # temporary disabled
    # def test_endpoint_return_list_of_topics_empty_if_topics(self):

    #     api.content.create(
    #         container=self.portal, type="Pagina Argomento", title="foo"
    #     )
    #     api.content.create(
    #         container=self.portal, type="Pagina Argomento", title="bar"
    #     )

    #     commit()
    #     response = self.api_session.get("/@search-filters").json()

    #     self.assertIn('topics', response)
    #     self.assertEqual(len(response['topics']), 2)



# -*- coding: utf-8 -*-
import logging

logger = logging.getLogger(__name__)

DEFAULT_PROFILE = "profile-design.plone.policy:default"


def update_profile(context, profile):
    context.runImportStepFromProfile(DEFAULT_PROFILE, profile)


def update_types(context):
    update_profile(context, "typeinfo")


def update_rolemap(context):
    update_profile(context, "rolemap")


def update_registry(context):
    update_profile(context, "plone.app.registry")


def update_catalog(context):
    update_profile(context, "catalog")


def update_controlpanel(context):
    update_profile(context, "controlpanel")



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from design.plone.policy.testing import (
    DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING,
)
from plone.restapi.testing import RelativeSession
from plone import api
from Products.MailHost.interfaces import IMailHost
from zope.component import getUtility

import transaction
import unittest


class SendActionFormEndpoint(unittest.TestCase):

    layer = DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.mailhost = getUtility(IMailHost)

        registry = getUtility(IRegistry)
        registry["plone.email_from_address"] = "info@test.org"
        registry["plone.email_from_name"] = u"Plone test site"

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        self.document = api.content.create(
            type="Document", title="Example context", container=self.portal,
        )
        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {"@type": "form"},
        }
        self.document_url = self.document.absolute_url()
        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

        # set default block
        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {"@type": "form"},
        }
        transaction.commit()

    def submit_form(self, data):
        url = "{}/@send-action-form".format(self.document_url)
        response = self.api_session.post(url, json=data,)
        transaction.commit()
        return response

    def test_email_not_send_if_block_id_is_not_given(self):
        response = self.submit_form(
            data={"from": "john@doe.com", "message": "Just want to say hi."},
        )
        transaction.commit()

        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(res["message"], "block_id mancante")

    def test_email_not_send_if_block_id_is_incorrect_or_not_present(self):
        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "message": "Just want to say hi.",
                "block_id": "unknown",
            },
        )
        transaction.commit()

        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            res["message"],
            'Blocco di tipo "form" con id "unknown" non trovato',
        )

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "message": "Just want to say hi.",
                "block_id": "text-id",
            },
        )
        transaction.commit()

        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            res["message"],
            'Blocco di tipo "form" con id "text-id" non trovato',
        )

    def test_email_not_send_if_block_id_is_correct_but_required_fields_missing(
        self,
    ):
        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "message": "Just want to say hi.",
                "block_id": "form-id",
            },
        )
        transaction.commit()

        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            res["message"],
            "Campo obbligatorio tra message, subject e from mancante",
        )

    def test_email_sent_with_site_recipient(self,):
        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "message": "Just want to say hi.",
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()

        self.assertEqual(response.status_code, 204)
        msg = self.mailhost.messages[0]
        if isinstance(msg, bytes) and bytes is not str:
            # Python 3 with Products.MailHost 4.10+
            msg = msg.decode("utf-8")
        self.assertIn("Subject: test subject", msg)
        self.assertIn("From: john@doe.com", msg)
        self.assertIn("To: info@test.org", msg)
        self.assertIn("Reply-To: john@doe.com", msg)
        self.assertIn("Just want to say hi.", msg)

    def test_email_sent_ignore_passed_recipient(self,):

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "to": "to@spam.com",
                "message": "Just want to say hi.",
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()

        self.assertEqual(response.status_code, 204)
        msg = self.mailhost.messages[0]
        if isinstance(msg, bytes) and bytes is not str:
            # Python 3 with Products.MailHost 4.10+
            msg = msg.decode("utf-8")
        self.assertIn("Subject: test subject", msg)
        self.assertIn("From: john@doe.com", msg)
        self.assertIn("To: info@test.org", msg)
        self.assertIn("Reply-To: john@doe.com", msg)
        self.assertIn("Just want to say hi.", msg)

    def test_email_sent_with_block_recipient_if_set(self,):

        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {"@type": "form", "to": "to@block.com"},
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "message": "Just want to say hi.",
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()

        self.assertEqual(response.status_code, 204)
        msg = self.mailhost.messages[0]
        if isinstance(msg, bytes) and bytes is not str:
            # Python 3 with Products.MailHost 4.10+
            msg = msg.decode("utf-8")
        self.assertIn("Subject: test subject", msg)
        self.assertIn("From: john@doe.com", msg)
        self.assertIn("To: to@block.com", msg)
        self.assertIn("Reply-To: john@doe.com", msg)
        self.assertIn("Just want to say hi.", msg)

    def test_email_sent_with_block_subject_if_set_and_not_passed(self,):

        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {"@type": "form", "default_subject": "block subject"},
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "message": "Just want to say hi.",
                "block_id": "form-id",
            },
        )
        transaction.commit()

        self.assertEqual(response.status_code, 204)
        msg = self.mailhost.messages[0]
        if isinstance(msg, bytes) and bytes is not str:
            # Python 3 with Products.MailHost 4.10+
            msg = msg.decode("utf-8")
        self.assertIn("Subject: block subject", msg)
        self.assertIn("From: john@doe.com", msg)
        self.assertIn("To: info@test.org", msg)
        self.assertIn("Reply-To: john@doe.com", msg)
        self.assertIn("Just want to say hi.", msg)



# -*- coding: utf-8 -*-
from copy import deepcopy
from plone import api
from plone.memoize import ram
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.services import Service
from time import time
from urllib.parse import urlencode
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse

import logging
import requests
import six

logger = logging.getLogger(__name__)
ENDPOINT = "https://api.twitter.com/2/tweets/search/recent"


def _feed_cachekey(method, self, query):
    """
    method for ramcache that store time and query.
    Cache time is 30 minutes
    """
    timestamp = time() // (60 * 30 * 1)
    return "{timestamp}:{query}".format(
        timestamp=timestamp, query=urlencode(query)
    )


@implementer(IPublishTraverse)
class TwitterFeedGet(Service):
    def reply(self):

        token = api.portal.get_registry_record(
            name="design.plone.policy.twitter_token"
        )
        if not token:
            msg = "BEARER token not set: please set it into plone registry to be able to fetch Tweets."  # noqa
            logger.error(msg)
            self.request.response.setStatus(500)
            return dict(error=dict(message=msg))

        query = self.generate_query()
        if query.get("error", ""):
            self.request.response.setStatus(400)
            return dict(error=dict(message=query["error"]))
        try:
            res = self.retrieve_tweets(query=query)
        except Exception as e:
            logger.exception(e)
            self.request.response.setStatus(500)
            msg = getattr(e, "message", e.__str__())
            return dict(error=dict(message=msg))
        return res

    @ram.cache(_feed_cachekey)
    def retrieve_tweets(self, query):
        token = api.portal.get_registry_record(
            name="design.plone.policy.twitter_token"
        )

        resp = requests.get(
            url=ENDPOINT,
            params=query,
            headers={"Authorization": "Bearer {}".format(token)},
        )
        # raise an exception if resp is not successful
        resp.raise_for_status()
        return self.convert_tweets(data=resp.json())

    def generate_query(self):
        query = self.request.form.copy()
        query = unflatten_dotted_dict(query)
        if not query:
            return {"error": "You need to provide at least an author."}
        max_results = int(query.get("max_results", "10"))
        if max_results < 10 or max_results > 100:
            return {"error": "max_results should be between 10 and 100."}
        authors = query.get("authors", [])
        if not authors:
            return {"error": "You need to provide at least an author."}
        if isinstance(authors, six.string_types):
            authors = [authors]
        res = {
            "query": " OR ".join(["from: {}".format(x) for x in authors]),
            # additional infos for tweets
            "tweet.fields": "entities,source,public_metrics,created_at",
            "expansions": "attachments.media_keys,author_id",
            "media.fields": "type,preview_image_url,height,media_key,public_metrics,url,width",  # noqa
            "user.fields": "name,profile_image_url,username",
        }
        return res

    def convert_tweets(self, data):
        tweets = data.get("data", [])
        users = data.get("includes", {}).get("users", [])

        if not tweets:
            return []

        res = []
        for tweet in tweets:
            html = self.convert_data_to_html(data=tweet)
            if html:
                tweet_data = {
                    "text": html,
                    "id": tweet["id"],
                    "created_at": tweet["created_at"],
                    "retweet_count": tweet["public_metrics"]["retweet_count"],
                    "reply_count": tweet["public_metrics"]["reply_count"],
                    "like_count": tweet["public_metrics"]["like_count"],
                }
                author_id = tweet.get("author_id", "")
                for user in users:
                    if author_id == user["id"]:
                        tweet_data["author"] = user
                res.append(tweet_data)
        return res

    def convert_data_to_html(self, data):
        entities = data.get("entities", {})
        text = data.get("text")
        if not entities:
            return text
        html = deepcopy(text)
        href_template = (
            '<a href="{url}" title="{title}" target="_blank">{text}</a>'
        )
        #  replace hashtags
        for hashtag in entities.get("hashtags", []):
            tag_text = text[hashtag["start"] : hashtag["end"]]  # noqa
            replaced = href_template.format(
                url="https://twitter.com/hashtag/{}".format(hashtag["tag"]),
                title=tag_text,
                text=tag_text,
            )
            html = html.replace(tag_text, replaced,)

        # replace mentions
        for mention in entities.get("mentions", []):
            mention_text = text[mention["start"] : mention["end"]]  # noqa
            replaced = href_template.format(
                url="https://twitter.com/{}".format(mention["username"]),
                title=mention_text,
                text=mention_text,
            )
            html = html.replace(mention_text, replaced,)

        #  replace urls
        occurrences = {}
        for url_data in entities.get("urls", []):
            url = url_data["url"]
            if url not in occurrences:
                occurrences[url] = 1
            else:
                occurrences[url] += 1
        for url_data in entities.get("urls", []):
            url = url_data["url"]
            to_replace = text[url_data["start"] : url_data["end"]]  # noqa
            if occurrences[url] > 1:
                html = html.replace(to_replace, "")
            else:
                if url_data["display_url"].startswith("pic.twitter.com"):
                    html = html.replace(to_replace, "")
                else:
                    replaced = href_template.format(
                        url=url,
                        title=url_data.get("title", url_data["display_url"]),
                        text=url_data["display_url"],
                    )
                    html = html.replace(to_replace, replaced,)
        return html
